@@ -1,8 +1,10 @@
 package Reddit::Client;
 
-our $VERSION = '1.10';
-# 1.10 added approve and ignore reports functions
-# 1.0991: there was a debug message and get_permalink was broken  
+our $VERSION = '1.20';
+# 1.20 web type apps now supported!
+# get_wiki, get_wiki_data
+## edit_wiki now takes alias 'sub'
+# dropped token refresh time to 3595 to account for lag or other fuckery
 
 $VERSION = eval $VERSION;
 
@@ -11,7 +13,7 @@ use warnings;
 use Carp;
 
 use Data::Dumper   qw/Dumper/;
-use JSON           qw//;
+use JSON           qw/decode_json/;
 use File::Spec     qw//;
 use Digest::MD5    qw/md5_hex/;
 use POSIX          qw/strftime/;
@@ -107,6 +109,7 @@ use constant API_EDIT                => 33;
 use constant API_REMOVE              => 34;
 use constant API_APPROVE             => 35;
 use constant API_IGNORE_REPORTS      => 36;
+use constant API_GETWIKI             => 37;
 
 #===============================================================================
 # Parameters
@@ -146,6 +149,7 @@ $API[API_USER          ] = ['GET',  '/user/%s/%s'             ];
 $API[API_SELECTFLAIR   ] = ['POST', '/r/%s/api/selectflair'   ];
 $API[API_FLAIROPTS     ] = ['POST', '/r/%s/api/flairselector' ];
 $API[API_EDITWIKI      ] = ['POST', '/r/%s/api/wiki/edit'     ];
+$API[API_GETWIKI       ] = ['GET',  '/r/%s/wiki/%s'           ];
 $API[API_CREATEMULTI   ] = ['POST', '/api/multi/user/%s/m/%s' ];
 $API[API_GETMULTI      ] = ['GET', '/api/multi/user/%s/m/%s%s'];
 $API[API_DELETEMULTI   ] = ['DELETE','/api/multi/user/%s/m/%s'];
@@ -201,11 +205,13 @@ use fields (
     	'tokentype',    # unused but saved for reference
     	'last_token',	# time last token was acquired
     	'client_id',	# These 4 values saved for automatic token refreshing
-    	'secret',
+    	'secret',       
 	'username',
 	'password',
-	'request_errors',
-	'print_response',
+	'request_errors',# print request errors
+	'print_response',# print entire response
+	'refresh_token', # oauth refresh token
+	'auth_type',
 );
 
 sub new {
@@ -213,24 +219,38 @@ sub new {
 	my $self = fields::new($class);
 
 	if (not exists $param{user_agent}) {
-		carp "Reddit::Client->new: user_agent required in future version.";
-		$param{user_agent} = $UA;
+		 croak "param 'user_agent' is required.";
 	}
 	$self->{user_agent} 	= $param{user_agent};
 	$self->{request_errors} = $param{request_errors} || 0;
 	$self->{print_response} = $param{print_response} || 0;
 
-	if ($param{username} || $param{password} || $param{client_id} || $param{secret}) {
-		if (!$param{username} || !$param{password} || !$param{client_id} || !$param{secret}) {
-			croak "If any of username, password, client_id, or secret are provided, all are required.";
+	if ($param{username} or $param{password}) {
+		if (!$param{username} or !$param{password}) {
+			croak "If username or  password are provided, both are required.";
+		} elsif (!$param{client_id} or !$param{secret}) {
+			croak "client_id and secret are required for authorized apps.";
 		} else {
-			$self->get_token(
-					client_id	=> $param{client_id},
-					secret		=> $param{secret},
-					username	=> $param{username},
-					password	=> $param{password},	
-			);
+			$self->{auth_type} 	= 'script';
+			$self->{client_id}	= $param{client_id};
+			$self->{secret} 	= $param{secret};
+			$self->{username}	= $param{username};
+			$self->{password}	= $param{password};
+
+			$self->get_token();
 		}
+	} elsif ($param{refresh_token}) {
+			croak "client_id and secret are required for authorized apps." unless $param{client_id} and $param{secret}; 
+		
+		$self->{auth_type} 	= 'webapp';
+		$self->{client_id}	= $param{client_id};
+		$self->{secret} 	= $param{secret};
+		$self->{refresh_token} 	= $param{refresh_token};
+
+		$self->get_token();
+	} else {
+		# can this even be run without auth anymore?
+		$self->{auth_type} = 'none';
 	}
 
     return $self;
@@ -248,8 +268,13 @@ sub version {
 sub request {
     my ($self, $method, $path, $query, $post_data) = @_;
 
-	if (!$self->{last_token} || $self->{last_token} <= time - 3598) {
-		$self->get_token(client_id=>$self->{client_id}, secret=>$self->{secret}, username=>$self->{username}, password=>$self->{password});
+	if (!$self->{last_token} || $self->{last_token} <= time - 3595) {
+		$self->get_token(
+			client_id	=> $self->{client_id}, 
+			secret		=> $self->{secret}, 
+			username	=> $self->{username}, 
+			password	=> $self->{password},
+		);
 	}
 
     # Trim leading slashes off of the path
@@ -275,15 +300,46 @@ sub request {
 
 sub get_token {
 	my ($self, %param) = @_;
-	$self->{client_id} 	= $param{client_id} || croak "need client_id"; 
-	$self->{secret} 	= $param{secret}    || croak "need secret";
-	$self->{username}	= $param{username}  || croak "need username";
-	$self->{password}	= $param{password}  || croak "need password";
+
+	# let people set auth things here. this was stupid to allow.
+	if ($param{username} or $param{password}) {
+		die "get_token: if username or password are provided, all 4 script-type authentication arguments (username, password, client_id, secret) are required." unless $param{username} and $param{password} and $param{client_id} and $param{secret}; 
+
+		$self->{auth_type} 	= 'script';
+		$self->{client_id}	= $param{client_id};
+		$self->{secret} 	= $param{secret};
+		$self->{username}	= $param{username};
+		$self->{password}	= $param{password};
+
+	} elsif ($param{refresh_token}) {
+		die "get_token: if refresh_token is provided, all 3 webapp-type authenticaltion arguments (refresh_token, client_id, secret) are required." unless $param{refresh_token} and $param{client_id} and $param{secret};
+
+		$self->{auth_type} 	= 'webapp';
+		$self->{client_id}	= $param{client_id};
+		$self->{secret} 	= $param{secret};
+		$self->{refresh_token} 	= $param{refresh_token};
+	}
+
 	$self->{last_token} 	= time;
 
-	my $message = Reddit::Client::Request->token_request($self->{client_id}, $self->{secret}, $self->{username}, $self->{password}, $self->{user_agent});
+	# why don't we just pass in the whole Client object ffs
+	my %p = (
+		client_id	=> $self->{client_id},
+		secret		=> $self->{secret},
+		user_agent	=> $self->{user_agent},
+		auth_type	=> $self->{auth_type},
+	);
+
+	if ($self->{auth_type} eq 'script') {
+		$p{username} = $self->{username},
+		$p{password} = $self->{password},
+	} elsif ($self->{auth_type} eq 'webapp') {
+		$p{refresh_token} = $self->{refresh_token};
+	} else { die "get_token: invalid auth type"; }
+
+	my $message = Reddit::Client::Request->token_request(%p);
 	#dd $message;
-	my $j = JSON::decode_json($message);
+	my $j = decode_json($message);
 	$self->{token} 		= $j->{access_token};
 	$self->{tokentype} 	= $j->{token_type};
 
@@ -292,7 +348,7 @@ sub get_token {
 
 sub has_token {
 	my $self = shift;
-	return (!$self->{last_token} || $self->{last_token} <= time - 3598) ? 0 : 1;
+	return (!$self->{last_token} || $self->{last_token} <= time - 3595) ? 0 : 1;
 }
 
 sub json_request {
@@ -919,13 +975,36 @@ sub get_flair_options {
 # Subreddit management
 #==============================================================================
 
+sub get_wiki {
+	my ($self, %param) = @_;
+	my $page 	= $param{page} || croak "Need 'page'";
+	my $sub 	= $param{sub} || $param{subreddit} || die "need subreddit\n";
+
+	my $data 	= {};
+	$data->{v}	= $param{v}   if $param{v};
+	$data->{v2}	= $param{v2}  if $param{v2};
+
+	
+	my $result = $self->api_json_request(
+		api 	=> API_GETWIKI,
+		args 	=> [$sub, $page],
+		data	=> $data,
+	);
+	return $param{data} ? $result->{data} : $result->{data}->{content_md};
+}
+sub get_wiki_data {
+	my ($self, %param) = @_;
+	$param{data} = 1;
+	return $self->get_wiki(%param);
+}
+
 sub edit_wiki {
 	my ($self, %param) = @_;
 	my $page 	= $param{page} || croak "Need 'page'";
 	my $content	= defined $param{content} ? $param{content} : croak "Need 'content'";
 	# Reddit maximum length is 524,288
 	if (length $content > 524288) { croak "Maximum length for 'content' is 524288 bytes."; }
-	my $sub		= $param{subreddit} || croak "Need 'subreddit'";
+	my $sub		= $param{sub} || $param{subreddit} || croak "Need 'sub' or 'subreddit'";
 	my $previous	= $param{previous};
 	my $reason	= $param{reason};
 
