@@ -34,6 +34,11 @@ sub edit {
 	$self->{selftext} = $text if $post;
 	return $post;
 }
+sub delete {
+    	my $self = shift;
+	my $cmtid = $self->{session}->delete($self->{name});
+	return $cmtid;
+}
 sub comments {
     my $self = shift;
     return $self->{session}->get_comments(permalink => $self->{permalink});
