@@ -8,6 +8,7 @@ use LWP::UserAgent qw//;
 use HTTP::Request  qw//;
 use URI::Encode    qw/uri_encode/;
 use URI::Escape    qw/uri_escape/; # next update, also line 122
+use Data::Dump::Color;
 
 require Reddit::Client;
 
@@ -114,19 +115,27 @@ sub send {
 }
 
 sub token_request {
-	my ($self, $client_id, $secret, $username, $password, $useragent) = @_;
+	my ($self, %param) = @_;
 
-	my $url = "https://$client_id:$secret\@www.reddit.com/api/v1/access_token";
+	my $url = "https://$param{client_id}:$param{secret}\@www.reddit.com/api/v1/access_token";
 
-    	my $ua = LWP::UserAgent->new(agent => $useragent);
+    	my $ua = LWP::UserAgent->new(agent => $param{user_agent});
 	my $req = HTTP::Request->new(POST => $url);
 	$req->header('content-type' => 'application/x-www-form-urlencoded');
 
 	#my $postdata = "grant_type=password&username=$username&password=$password";
-	my $postdata = "grant_type=password&username=$username&password=" . uri_escape($password);
+	my $postdata;
+		
+	if ($param{auth_type} eq 'script') {
+		$postdata = "grant_type=password&username=$param{username}&password=" . uri_escape($param{password});
+	} elsif ($param{auth_type} eq 'webapp') {
+		$postdata = "grant_type=refresh_token&refresh_token=".uri_escape($param{refresh_token});
+	} else { die "Request:token_request: invalid auth type"; }
+
 	$req->content($postdata);
 
     	my $res = $ua->request($req);
+	#dd $res;
 
     	if ($res->is_success) {
         	return $res->decoded_content;
